"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateScenarioStructure = exports.deactivate = exports.activate = exports.countTotalDuration = exports.countDialogueCharacters = void 0;
const vscode = require("vscode");
const fs = require("fs");
const path = require("path");
const os = require("os");
const inputRange_1 = require("./utils/inputRange");
const countDialogueCharactersInRange_1 = require("./utils/countDialogueCharactersInRange");
const dateUtils_1 = require("./utils/dateUtils");
const templateLoader_1 = require("./utils/templateLoader");
const countTotalDurationInRange_1 = require("./utils/countTotalDurationInRange");
const formatDuration_1 = require("./utils/formatDuration");
const parseYamlWithFeedback_1 = require("./utils/parseYamlWithFeedback");
/////////////////////////////////////////////////////// 文字数カウント
async function countDialogueCharacters() {
    const editor = vscode.window.activeTextEditor;
    if (!editor) {
        vscode.window.showErrorMessage('ファイルを開いてください。');
        return;
    }
    const text = editor.document.getText();
    const yamlData = (0, parseYamlWithFeedback_1.parseYamlWithFeedback)(text);
    if (!yamlData?.sequences) {
        vscode.window.showErrorMessage('シナリオ構造が不正です。sequencesが見つかりません。');
        return;
    }
    const sequenceIds = yamlData.sequences.map((seq) => seq.id);
    const minSequenceId = Math.min(...sequenceIds);
    const maxSequenceId = Math.max(...sequenceIds);
    const ranges = await (0, inputRange_1.promptSequenceAndSceneRange)(minSequenceId, maxSequenceId);
    if (!ranges)
        return;
    const { sequenceRange, sceneRange } = ranges;
    try {
        const count = (0, countDialogueCharactersInRange_1.countDialogueCharactersInRange)(text, sequenceRange, sceneRange);
        const estimatedSeconds = Math.ceil(count / 5); // 5文字 = 1秒（仮定）
        vscode.window.showInformationMessage(`セリフ文字数: ${count} 文字（約 ${estimatedSeconds} 秒）`);
    }
    catch (err) {
        vscode.window.showErrorMessage(err.message || 'セリフ文字数カウントに失敗しました。');
    }
}
exports.countDialogueCharacters = countDialogueCharacters;
/////////////////////////////////////////////////////// 総尺カウントコマンド
async function countTotalDuration() {
    const editor = vscode.window.activeTextEditor;
    if (!editor) {
        vscode.window.showErrorMessage('ファイルを開いてください。');
        return;
    }
    const text = editor.document.getText();
    const yamlData = (0, parseYamlWithFeedback_1.parseYamlWithFeedback)(text);
    if (!yamlData?.sequences) {
        vscode.window.showErrorMessage('sequencesが見つかりません。');
        return;
    }
    const sequenceIds = yamlData.sequences.map((seq) => seq.id);
    const minSequenceId = Math.min(...sequenceIds);
    const maxSequenceId = Math.max(...sequenceIds);
    const ranges = await (0, inputRange_1.promptSequenceAndSceneRange)(minSequenceId, maxSequenceId);
    if (!ranges)
        return;
    const { sequenceRange, sceneRange } = ranges;
    try {
        const total = (0, countTotalDurationInRange_1.countTotalDurationInRange)(text, sequenceRange, sceneRange);
        const formatted = (0, formatDuration_1.formatDuration)(total);
        vscode.window.showInformationMessage(`総尺: ${total}秒（${formatted}）`);
    }
    catch (err) {
        vscode.window.showErrorMessage(err.message || '総尺カウントに失敗しました。');
    }
}
exports.countTotalDuration = countTotalDuration;
function activate(context) {
    /////////////////////////////////////////////////////// Add NewFile
    context.subscriptions.push(vscode.commands.registerCommand('sp.createNewFile', async () => {
        const now = (0, dateUtils_1.getTodayIsoString)();
        const title = await vscode.window.showInputBox({
            prompt: '新しい作品のタイトルを入力してください',
            placeHolder: '例: 放課後、そして未来へ',
            value: '新しい作品タイトル'
        });
        if (title === undefined)
            return;
        const authorsInput = await vscode.window.showInputBox({
            prompt: '作者名をカンマ区切りで入力してください（例: 山田太郎, 佐藤花子）',
            placeHolder: '作者名を入力（複数可）',
            value: '作者名'
        });
        if (authorsInput === undefined)
            return;
        const authorsList = authorsInput.split(',').map(a => a.trim()).filter(a => a.length > 0);
        const authorsYaml = authorsList.map(a => `  - "${a}"`).join('\n');
        const content = (0, templateLoader_1.loadScenarioTemplate)(context, {
            title,
            authors: authorsYaml,
            created_at: now,
            updated_at: now
        });
        const timestamp = new Date().getTime();
        const uri = vscode.Uri.parse(`untitled:新しいシナリオ-${timestamp}.sp.yml`);
        const edit = new vscode.WorkspaceEdit();
        edit.createFile(uri, { ignoreIfExists: true });
        await vscode.workspace.applyEdit(edit);
        const doc = await vscode.workspace.openTextDocument(uri);
        const editor = await vscode.window.showTextDocument(doc, { preview: false });
        await editor.edit(builder => builder.insert(new vscode.Position(0, 0), content));
    }));
    /////////////////////////////////////////////////////// Add Template
    context.subscriptions.push(vscode.commands.registerCommand('sp.addTemplate', async () => {
        const editor = vscode.window.activeTextEditor;
        if (!editor) {
            vscode.window.showErrorMessage('エディターが開かれていません。');
            return;
        }
        const today = (0, dateUtils_1.getTodayIsoString)();
        const content = (0, templateLoader_1.loadScenarioTemplate)(context, {
            title: "タイトルをここに",
            authors: '  - "作者"',
            created_at: today,
            updated_at: today
        });
        const position = editor.selection.active;
        await editor.edit(builder => builder.insert(position, `\n${content}\n`));
    }));
    /////////////////////////////////////////////////////// Add Sequences
    context.subscriptions.push(vscode.commands.registerCommand('sp.addSequences', async () => {
        const editor = vscode.window.activeTextEditor;
        if (!editor) {
            vscode.window.showErrorMessage('エディターが開かれていません。');
            return;
        }
        const position = editor.selection.active;
        const lineText = editor.document.lineAt(position.line).text;
        const baseIndent = ' '.repeat(2); // 2スペース
        const snippet = `
${baseIndent}- id: 
${baseIndent}  name: ""
${baseIndent}  description: ""
${baseIndent}  scenes: 
  `;
        await editor.edit(editBuilder => {
            editBuilder.insert(position, snippet);
        });
    }));
    /////////////////////////////////////////////////////// Add Scene
    context.subscriptions.push(vscode.commands.registerCommand('sp.addScene', async () => {
        const editor = vscode.window.activeTextEditor;
        if (!editor) {
            vscode.window.showErrorMessage('アクティブなドキュメントがありません');
            return;
        }
        const position = editor.selection.active;
        const newPosition = position.with(position.line, 0);
        editor.selection = new vscode.Selection(newPosition, newPosition);
        const baseIndent = ' '.repeat(6); // 6スペース
        const snippet = `
${baseIndent}- id: 
${baseIndent}  name: "新しいシーン"
${baseIndent}  duration: 
${baseIndent}  time_of_day: "昼"
${baseIndent}  se: 
${baseIndent}  description: ""
${baseIndent}  actors: 
${baseIndent}  cuts:
${baseIndent}    - id: 
${baseIndent}      camera: ""
${baseIndent}      description: ""
${baseIndent}      se: 
${baseIndent}      dialogue: 
${baseIndent}      duration: 3
${baseIndent}      transition: 
`;
        editor.insertSnippet(new vscode.SnippetString(snippet), editor.selection.active);
    }));
    /////////////////////////////////////////////////////// Add Cut
    context.subscriptions.push(vscode.commands.registerCommand('sp.addCut', async () => {
        const editor = vscode.window.activeTextEditor;
        if (!editor) {
            vscode.window.showErrorMessage('アクティブなドキュメントがありません');
            return;
        }
        const position = editor.selection.active;
        const newPosition = position.with(position.line, 0);
        editor.selection = new vscode.Selection(newPosition, newPosition);
        // 固定インデントでスニペット作成（例：10スペース）
        const baseIndent = ' '.repeat(10);
        const snippet = `
${baseIndent}- id: 
${baseIndent}  camera: ""
${baseIndent}  description: ""
${baseIndent}  se: 
${baseIndent}  dialogue: 
${baseIndent}  duration: 3
${baseIndent}  transition: 
`;
        editor.insertSnippet(new vscode.SnippetString(snippet), newPosition);
    }));
    ///// Count Total Duration
    context.subscriptions.push(vscode.commands.registerCommand('sp.countTotalDuration', countTotalDuration));
    ///// Count Words
    context.subscriptions.push(vscode.commands.registerCommand('sp.countDialogueCharacters', countDialogueCharacters));
    /////////////////////////////////////////////////////// Export Html&CSS
    context.subscriptions.push(vscode.commands.registerCommand('sp.exportHtml', async () => {
        const editor = vscode.window.activeTextEditor;
        if (!editor) {
            vscode.window.showErrorMessage('出力対象が選択されていません');
            return;
        }
        const document = editor.document;
        const text = document.getText();
        const yamlContent = (0, parseYamlWithFeedback_1.parseYamlWithFeedback)(text);
        if (!yamlContent) {
            vscode.window.showErrorMessage('YAMLが空です。');
            return;
        }
        const layoutAndCss = await selectLayoutAndGenerateCssFileName();
        if (!layoutAndCss)
            return;
        const { layout, cssFileName } = layoutAndCss;
        const safeTitle = (yamlContent.title || "scenario")
            .replace(/\s+/g, "_")
            .replace(/[\\/:*?"<>|]/g, "_");
        const defaultFileName = `${safeTitle}.html`;
        const workspaceFolder = vscode.workspace.workspaceFolders?.[0]?.uri.fsPath || os.homedir();
        const uri = await vscode.window.showSaveDialog({
            defaultUri: vscode.Uri.file(path.join(workspaceFolder, defaultFileName)),
            filters: { 'HTML Files': ['html'] }
        });
        if (!uri) {
            vscode.window.showInformationMessage('書き出しがキャンセルされました。');
            return;
        }
        const styleUri = vscode.Uri.file(path.join(path.dirname(uri.fsPath), 'style.css'));
        const html = generateHtml(yamlContent, styleUri);
        fs.writeFileSync(uri.fsPath, html, 'utf8');
        const extensionPath = context.extensionPath;
        const cssSourcePath = path.join(extensionPath, 'media', cssFileName);
        const cssTargetPath = path.join(path.dirname(uri.fsPath), 'style.css');
        fs.copyFile(cssSourcePath, cssTargetPath, async (err) => {
            if (err) {
                console.error('CSSコピーエラー:', err); // エラー内容をログに出力
                vscode.window.showErrorMessage('CSSコピーに失敗しました。詳細はコンソールを確認してください。');
                return;
            }
            vscode.window.showInformationMessage(`HTMLとCSS（${layout}）を書き出しました。`);
            await vscode.env.openExternal(uri);
        });
    }));
    /////////////////////////////////////////////////////// Preview
    context.subscriptions.push(vscode.commands.registerCommand('sp.previewScene', () => {
        previewScene(context);
    }));
    vscode.window.showInformationMessage('ScreenPlay Formater 拡張機能が有効になりました！');
}
exports.activate = activate;
function deactivate() { }
exports.deactivate = deactivate;
async function previewScene(context) {
    const editor = vscode.window.activeTextEditor;
    if (!editor) {
        vscode.window.showErrorMessage("アクティブなドキュメントがありません");
        return;
    }
    const text = editor.document.getText();
    const parsed = (0, parseYamlWithFeedback_1.parseYamlWithFeedback)(text); // 差し替え
    if (!parsed)
        return; // パース失敗時は中断
    const panel = vscode.window.createWebviewPanel("scenePreview", "シナリオプレビュー", vscode.ViewColumn.Beside, { enableScripts: true });
    // path.join で media/style.css を作成
    const layoutAndCss = await selectLayoutAndGenerateCssFileName();
    if (!layoutAndCss)
        return;
    const { layout, cssFileName } = layoutAndCss;
    const localStylePath = vscode.Uri.file(path.join(context.extensionPath, 'media', cssFileName));
    // file: URI に変換
    const styleUri = panel.webview.asWebviewUri(localStylePath);
    // 生成HTMLに渡す
    const html = generateHtml(parsed, styleUri);
    panel.webview.html = html;
}
/////////////////////////////////////////////////////// HTML生成関数
function generateHtml(data, styleUri) {
    const structureError = validateScenarioStructure(data);
    if (structureError) {
        vscode.window.showErrorMessage(`シナリオ構造エラー: ${structureError}`);
        return `<h2>シナリオ構造エラー: ${structureError}</h2>`;
    }
    if (!data || !data.sequences) {
        return "<h2>シナリオが正しく構造化されていません</h2>";
    }
    let html = `
  <html>
  <head>
    <meta charset="UTF-8" />
    <link href="${styleUri}" rel="stylesheet" />
    <title>${data.title || "（タイトル未設定）"}</title>
  </head>
  <body>
  `;
    html += `<div class="scenario-header"><div class="scenario-title">${data.title || "（タイトル未設定）"}</div>`;
    html += `<div class="scenario-meta">`;
    if (data.authors) {
        html += `<div><strong>作者:</strong> ${Array.isArray(data.authors) ? data.authors.join(", ") : data.authors}</div>`;
    }
    if (data.created_at) {
        html += `<div><strong>作成日:</strong> ${data.created_at}</div>`;
    }
    if (data.updated_at) {
        html += `<div><strong>更新日:</strong> ${data.updated_at}</div>`;
    }
    if (data.version) {
        html += `<div><strong>バージョン:</strong> ${data.version}</div>`;
    }
    if (data.description) {
        html += `<div><strong>説明:</strong><br>${data.description.replace(/\n/g, "<br>")}</div>`;
    }
    if (data.repository) {
        html += `<div><strong>リポジトリ:</strong> <a href="${data.repository}" target="_blank">${data.repository}</a></div>`;
    }
    html += `</div></div>`;
    for (const seq of data.sequences) {
        html += `<div class="sequence" data-id="${seq.id}">
      <div class="sequence-header"><span class="label">シーケンス</span>${seq.id} ${seq.name}</div>
      <div class="sequence-description">${seq.description || ""}</div>`;
        for (const scene of seq.scenes || []) {
            html += `<div class="scene" data-id="${scene.id}">
        <div class="scene-header"><span class="label">シーン</span>${scene.id} ${scene.name}</div>
        <div class="scene-description">${scene.description || ""}</div>
        <div class="scene-time">${scene.time_of_day || "時間未設定"}</div>`;
            for (const cut of scene.cuts || []) {
                html += `<div class="cut" data-id="${cut.id}">
          <div class="cut-header"><span class="label">カット</span>${cut.id}</div>
          <div class="camera"><span class="label">カメラ:</span>${cut.camera || ""}</div>
          <div class="cut-description"><span class="label">描写:</span>${cut.description || ""}</div>
          <div class="cut-duration"><span class="label">時間:</span>${cut.duration || "?"}秒</div>
          <div class="dialogue"><span class="label">セリフ:</span>`;
                for (const d of cut.dialogue || []) {
                    html += `<div><span class="speaker">${d.speaker}:</span> <span class="text">${d.text}</span> (${d.description || ""})</div>`;
                }
                html += `</div>
          <div class="transition"><span class="label">トランジション:</span>${cut.transition || ""}</div>
        </div>`;
            }
            html += `</div>`; // scene
        }
        html += `</div>`; // sequence
    }
    html += "</body></html>";
    return html;
}
function renderBlock(data, fields, className) {
    const parts = [`<div class="${className}">`];
    for (const key in fields) {
        parts.push(`<div class="${key}">${fields[key]}: ${data[key] ?? ""}</div>`);
    }
    parts.push(`</div>`);
    return parts.join("\n");
}
/////////////////////////////////////////////////////// 構造検証関数
const schemaRules = [
    { path: "sequences", type: "array", required: true },
    { path: "sequences[].id", type: "number", required: true },
    { path: "sequences[].name", type: "string", required: true },
    { path: "sequences[].scenes", type: "array", required: true },
    { path: "sequences[].scenes[].id", type: "number", required: true },
    { path: "sequences[].scenes[].name", type: "string", required: true },
    { path: "sequences[].scenes[].cuts", type: "array", required: true },
    { path: "sequences[].scenes[].cuts[].id", type: "number", required: true },
    { path: "sequences[].scenes[].cuts[].dialogue", type: "array", required: false },
    { path: "sequences[].scenes[].cuts[].dialogue[].speaker", type: "string", required: true },
    { path: "sequences[].scenes[].cuts[].dialogue[].text", type: "string", required: true }
];
function validateScenarioStructure(data) {
    for (const rule of schemaRules) {
        const segments = rule.path.split('.');
        const error = resolvePathAndValidate(data, segments, rule.type, rule.required, rule.path);
        if (error)
            return error;
    }
    return null;
}
exports.validateScenarioStructure = validateScenarioStructure;
function resolvePathAndValidate(obj, segments, type, required, fullPath) {
    const segment = segments[0];
    const rest = segments.slice(1);
    const isArray = segment.endsWith("[]");
    const key = isArray ? segment.slice(0, -2) : segment;
    if (!(key in obj)) {
        return required ? `${fullPath} が存在しません` : null;
    }
    const value = obj[key];
    if (isArray) {
        if (!Array.isArray(value))
            return `${fullPath} は配列である必要があります`;
        for (let i = 0; i < value.length; i++) {
            const error = rest.length > 0
                ? resolvePathAndValidate(value[i], rest, type, required, `${fullPath.replace('[]', `[${i}]`)}`)
                : validateLeafType(value[i], type, `${fullPath}[${i}]`);
            if (error)
                return error;
        }
    }
    else {
        if (rest.length > 0) {
            return resolvePathAndValidate(value, rest, type, required, fullPath);
        }
        else {
            return validateLeafType(value, type, fullPath);
        }
    }
    return null;
}
function validateLeafType(value, type, path) {
    if (type === 'array') {
        if (!Array.isArray(value))
            return `${path} は配列である必要があります`;
    }
    else {
        if (typeof value !== type)
            return `${path} は ${type} 型である必要があります`;
    }
    return null;
}
async function selectLayoutAndGenerateCssFileName() {
    const layout = await vscode.window.showQuickPick(["縦書き（脚本風）", "横書き（モダン）", "印刷用（白黒）"], { placeHolder: "レイアウトスタイルを選択してください" });
    if (!layout) {
        vscode.window.showInformationMessage("出力がキャンセルされました。");
        return null;
    }
    let cssFileName = "style-vertical.css"; // デフォルトのCSSファイル名
    if (layout.includes("横書き"))
        cssFileName = "style-horizontal.css";
    if (layout.includes("印刷用"))
        cssFileName = "style-print.css";
    return { layout, cssFileName };
}
//# sourceMappingURL=_extension%20-%20%E3%82%B3%E3%83%94%E3%83%BC.js.map